<?php

namespace Bydedi\Shipping\Chronopost;

use Bydedi\Shipping\Chronopost\Exception\ChronopostException;
use ChronopostServiceShipping;
use ChronopostStructCustomerValue;
use ChronopostStructEsdValue;
use ChronopostStructEsdValue3;
use ChronopostStructEsdWithRefClientValue;
use ChronopostStructHeaderValue;
use ChronopostStructShipperValue;
use ChronopostStructShipping;
use ChronopostStructShippingMultiParcel;
use ChronopostStructSkybillParamsValue;
use ChronopostStructSkybillParamsValueV2;
use ChronopostStructSkybillValue;
use ChronopostStructSkybillWithDimensionsValueV3;
use ChronopostWsdlClass;
use ChronopostStructShippingMultiParcelWithReservation;

/**
 * Class ShippingService
 * @package Bydedi\Shipping\Chronopost
 */
class ShippingService
{
    const WSDL = 'https://ws.chronopost.fr/shipping-cxf/ShippingServiceWS?wsdl';

    /**
     * @var string
     */
    protected $login;

    /**
     * @var string
     */
    protected $pass;

    /**
     * @var string
     */
    protected $ws_pass;

    /**
     * @var string
     */
    protected $dir;

    /**
     * @var array
     */
    protected $shipper_infos;

    /**
     * @var ChronopostServiceShipping
     */
    protected $chronopostServiceShipping;

    /**
     * @var string
     * voir readme pour les différentes valeur de cette variable
     */
    protected $format;

    /**
     * ShippingService constructor.
     *
     * @param $config
     *
     * @throws ChronopostException
     */
    public function __construct($config)
    {
        $this->login = $config['login'];
        $this->pass = $config['pass'];
        $this->ws_pass = $config['ws_pass'];
        $this->dir = $config['dir'];
        $this->shipper_infos = $config['shipper'];

        $this->format = (!empty($config['format'])) ? $config['format'] : 'PDF';

        $this->chronopostServiceShipping = new ChronopostServiceShipping($this->getSoapConfig());

        if (!is_dir($this->dir) && !mkdir($this->dir, 0775, true)) {
            throw new ChronopostException($this->chronopostServiceShipping, sprintf('The dir %s is not writable', $this->dir));
        }
    }

    /**
     * @param Sticker $sticker
     *
     * @return Sticker
     *
     * @throws ChronopostException
     */
    public function generate(Sticker $sticker)
    {

        if ($sticker->getNumberOfParcel() > 1) {
            $sticker = $this->generateMultiParcel($sticker);
        } else {
            $sticker = $this->generateSimple($sticker);
        }

        return $sticker;
    }

    protected function generateSimple(Sticker $sticker)
    {
        $shipper = $sticker->getShipper();
        if (empty($shipper)) {
            $shipper = $this->createDefaultShipper();
        }

        $customer = $sticker->getCustomer();
        if (empty($customer)) {
            $customer = $this->createDefaultCustomer();
        }

        $header = $this->getHeaderValue();

        $date = date('Y-m-d');

        $skybill = new ChronopostStructSkybillValue();
        $skybill->setBulkNumber('bulkNumber');
        $skybill->setCodValue('0');
        $skybill->setCodCurrency($sticker->getCurrency());
        $skybill->setCustomsCurrency($sticker->getCurrency());
        $skybill->setCustomsValue('0');
        $skybill->setEvtCode("DC");
        $skybill->setInsuredCurrency($sticker->getCurrency());
        $skybill->setInsuredValue('0');
        $skybill->setObjectType("MAR");
        $skybill->setObjectType("MAR");
        $skybill->setPortCurrency($sticker->getCurrency());
        $skybill->setPortValue($sticker->getPortValue());
        $skybill->setProductCode($sticker->getProductCode());
        $skybill->setService(0);
        $skybill->setShipDate(date('Y-m-d\TH:i:sP', strtotime($date . 'T14:30:00Z')));
        $skybill->setShipHour("10");
        $skybill->setWeight((int)$sticker->getWeight() / 1000);
        $skybill->setWeightUnit("KGM");

        $skybillParams = new ChronopostStructSkybillParamsValue(null, $this->format);

        $refEsdClient = new ChronopostStructEsdValue(
            date('Y-m-d\TH:i:sP', strtotime($date . 'T17:30:00Z')),
            10,
            10,
            date('Y-m-d\TH:i:sP', strtotime($date . 'T14:30:00Z')),
            "1",
            "",
            "",
            "",
            10
        );;

        $esdRefClient = new ChronopostStructEsdWithRefClientValue(0, 0);
        $esdRefClient->setHeight($refEsdClient->getHeight());
        $esdRefClient->setWidth($refEsdClient->getWidth());
        $esdRefClient->setLength($refEsdClient->getLength());

        $theMethod = new ChronopostStructShipping(
            $esdRefClient,
            $header,
            $shipper,
            $customer,
            $sticker->getRecipient(),
            $sticker->getRef(),
            $skybill,
            $skybillParams,
            $this->ws_pass
        );

        //appel de la fonction
        $result = $this->chronopostServiceShipping->shipping($theMethod);
        if (!$result) {
            throw new ChronopostException($this->chronopostServiceShipping, print_r($this->chronopostServiceShipping->getLastError(), true));
        }

        if (!empty($this->chronopostServiceShipping->getResult()->return->errorCode)) {
            throw new ChronopostException(
                $this->chronopostServiceShipping,
                sprintf(
                    '%s : %s',
                    $this->chronopostServiceShipping->getResult()->return->errorCode,
                    $this->chronopostServiceShipping->getResult()->return->errorMessage
                )
            );
        }

        if (empty($this->chronopostServiceShipping->getResult()->return->skybill)) {
            throw new ChronopostException($this->chronopostServiceShipping, 'Error, no field skybill');
        }

        $filename = sprintf(
            '%s.pdf',
            $this->chronopostServiceShipping->getResult()->return->skybillNumber
        );

        $filePath = $this->dir . $filename;
        $contentPDF = $this->chronopostServiceShipping->getResult()->return->skybill;

        file_put_contents($filePath, $contentPDF);

        $sticker
            ->addUrlPDF($filePath)
            ->addTracking($this->chronopostServiceShipping->getResult()->return->skybillNumber)
        ;

        return $sticker;
    }

    protected function generateMultiParcel(Sticker $sticker)
    {

        $shipper = $sticker->getShipper();
        if (empty($shipper)) {
            $shipper = $this->createDefaultShipper();
        }

        $customer = $sticker->getCustomer();
        if (empty($customer)) {
            $customer = $this->createDefaultCustomer();
        }

        $header = $this->getHeaderValue();

        $date = date('Y-m-d');


        $skybill = new ChronopostStructSkybillWithDimensionsValueV3();
        $skybill->setBulkNumber('bulkNumber');
        $skybill->setCodValue('0');
        $skybill->setCodCurrency($sticker->getCurrency());
        $skybill->setCustomsCurrency($sticker->getCurrency());
        $skybill->setCustomsValue('0');
        $skybill->setEvtCode("DC");
        $skybill->setInsuredCurrency($sticker->getCurrency());
        $skybill->setInsuredValue('0');
        $skybill->setObjectType("MAR");
        $skybill->setObjectType("MAR");
        $skybill->setPortCurrency($sticker->getCurrency());
        $skybill->setPortValue($sticker->getPortValue());
        $skybill->setProductCode($sticker->getProductCode());
        $skybill->setService(0);
        $skybill->setShipDate(date('Y-m-d\TH:i:sP', strtotime($date . 'T14:30:00Z')));
        $skybill->setShipHour("10");
        $skybill->setWeight((int)$sticker->getWeight() / 1000);
        $skybill->setWeightUnit("KGM");
        $skybill->setHeight(10);
        $skybill->setWidth(10);
        $skybill->setLength(10);

        $skybillParams = new ChronopostStructSkybillParamsValueV2(0);
        $skybillParams->setMode($this->format);

        $esdRefClient = new ChronopostStructEsdValue3();
        $esdRefClient->setHeight(10);
        $esdRefClient->setWidth(10);
        $esdRefClient->setLength(10);
        $esdRefClient->setLtAImprimerParChronopost(0);
        $esdRefClient->setNombreDePassageMaximum(0);

        //objet final
        $theMethod = new ChronopostStructShippingMultiParcel(
            $esdRefClient,
            $header,
            $shipper,
            $customer,
            $sticker->getRecipient(),
            $sticker->getRef(),
            $skybill,
            $skybillParams,
            $this->ws_pass
        );

        $theMethod->setNumberOfParcel($sticker->getNumberOfParcel());
        $theMethod->setMultiParcel('Y');

        //appel de la fonction
        $result = $this->chronopostServiceShipping->shippingMultiParcel($theMethod);

        if (!$result) {
            throw new ChronopostException($this->chronopostServiceShipping, print_r($this->chronopostServiceShipping->getLastError(), true));
        }

        if (!empty($this->chronopostServiceShipping->getResult()->return->errorCode)) {
            throw new ChronopostException(
                $this->chronopostServiceShipping,
                sprintf(
                    '%s : %s',
                    $this->chronopostServiceShipping->getResult()->return->errorCode,
                    $this->chronopostServiceShipping->getResult()->return->errorMessage
                )
            );
        }

        if (empty($this->chronopostServiceShipping->getResult()->return->resultMultiParcelValue)) {
            throw new ChronopostException($this->chronopostServiceShipping, "no stickers in resultMultiParcelValue");
        }

        foreach ($this->chronopostServiceShipping->getResult()->return->resultMultiParcelValue as $resultMultiParcelValue) {
            $filename = sprintf(
                '%s.pdf',
                $resultMultiParcelValue->skybillNumber
            );

            $filePath = $this->dir . $filename;
            $contentPDF = $resultMultiParcelValue->pdfEtiquette;

            file_put_contents($filePath, $contentPDF);

            $sticker
                ->addUrlPDF($filePath)
                ->addTracking($resultMultiParcelValue->skybillNumber)
            ;
        }

        return $sticker;
    }

    /**
     * @return ChronopostStructHeaderValue
     */
    protected function getHeaderValue()
    {
        return new ChronopostStructHeaderValue($this->login, 'CHRFR');
    }

    public function expedie($order_id = null)
    {
        // TODO : expedie la commande passé en parametre,
        // sinon toutes les commandes en cours de prepa avec un numero de colis
    }

    /**
     * @return array
     */
    protected function getSoapConfig()
    {
        return [
            ChronopostWsdlClass::WSDL_URL => self::WSDL,
            ChronopostWsdlClass::WSDL_CACHE_WSDL => WSDL_CACHE_NONE,
            ChronopostWsdlClass::WSDL_TRACE => true,
            ChronopostWsdlClass::WSDL_LOGIN => $this->login,
            ChronopostWsdlClass::WSDL_PASSWD => $this->ws_pass,
        ];
    }

    /**
     * @return ChronopostStructShipperValue
     */
    protected function createDefaultShipper()
    {
        return new ChronopostStructShipperValue(
            $this->shipper_infos['adresse_rue'],
            $this->shipper_infos['adresse_rue2'],
            $this->shipper_infos['adresse_ville'],
            $this->shipper_infos['civilite'],
            $this->shipper_infos['nom'],
            $this->shipper_infos['pays_code'],
            $this->shipper_infos['pays_nom'],
            $this->shipper_infos['email'],
            $this->shipper_infos['telephone'],
            $this->shipper_infos['societe'],
            "",
            $this->shipper_infos['telephone'],
            // permet de recevoir sur l'adresse mail de l'expéditeur (balise shipperEmail) le suivi de l'expédition.
            // La valeur doit être à 11 pour activer le Mail Automatique de Suivi
            11,
            $this->shipper_infos['adresse_cp']
        );
    }

    protected function createDefaultCustomer()
    {
        return new ChronopostStructCustomerValue(
            $this->shipper_infos['adresse_rue'],
            $this->shipper_infos['adresse_rue2'],
            $this->shipper_infos['adresse_ville'],
            $this->shipper_infos['civilite'],
            $this->shipper_infos['nom'],
            $this->shipper_infos['pays_code'],
            $this->shipper_infos['pays_nom'],
            $this->shipper_infos['email'],
            $this->shipper_infos['telephone'],
            $this->shipper_infos['societe'],
            "",
            $this->shipper_infos['telephone'],
            0,
            $this->shipper_infos['adresse_cp'],
            'Y'
        );
    }

    /**
     * @return ChronopostServiceShipping
     */
    public function getChronopostServiceShipping()
    {
        return $this->chronopostServiceShipping;
    }
}
